/*
 * File: AdvPassage.js
 * -------------------
 * This file defines a class that models a passage in Adventure.  Each
 * passage is characterized by the following properties:
 *
 * - The word that defines the direction of motion, such as "NORTH"
 * - The name of the room to which the passage leads
 * - The name of the object that serves as a key to taking the passage
 *
 * This file is supplied in its complete form primarily to give you
 * another example of a simple class definition.
 */

"use strict";

/*
 * Factory method: AdvPassage
 * Usage: let passage = AdvPassage(dir, room, key);
 * ------------------------------------------------
 * Creates an AdvPassage from the specified properties.  The key parameter
 * is optional.  If it is undefined the passage requires no key.
 */

class AdvPassage {

    private direction: string;
    private room: string;
    private key: string | undefined;

    constructor(direction: string, room: string, key?: string | undefined) {
        this.direction = direction;
        this.room = room;
        this.key = key;
    }

    /*
     * Method: getDirection
     * Usage: let dir = passage.getDirection();
     * ----------------------------------------
     * Returns the string form of the motion verb used to indicate the direction.
     */
    getDirection(): string {
        return this.direction;
    };

    /*
     * Method: getDestinationRoom
     * Usage: let room = passage.getDestinationRoom();
     * -----------------------------------------------
     * Returns the name of the room to which this passage leads.
     */
    getDestinationRoom(): string {
        return this.room;
    }

    /*
     * Method: getKey
     * Usage: let key = passage.getKey();
     * ----------------------------------
     * Returns the name of the object necessary to follow this passage.  If
     * there is no key, getKey returns the constant undefined.
     */
    getKey() {
        return this.key;
    }
}

/*
 * Function: readPassage
 * Usage: let passage = readPassage(passageXML);
 * ---------------------------------------------
 * Creates the passage data structure by reading the contents of an XML
 * <passage> tag.
 */

function readPassage(passageXML: Element): AdvPassage {
    let dir = passageXML.getAttribute("dir")!.toLowerCase();
    let room = passageXML.getAttribute("room")!;
    let key: string | null | undefined = passageXML.getAttribute("key");
    if (key === null || key === "") {
        key = undefined;
    } else {
        key = key.toLowerCase();
    }
    return new AdvPassage(dir, room, key);
}
